import module
import os, commands

class commitHook(module.baseModule):
	"""This is a base module for pre-commit hooks"""
	svn_commands=['commit']
	filesFailed=False
	
	def preCommand(self, args):
		files=self.filesToCommit(args)
		output=[]
		for tmpFile in files:
			output.append(self.checkFile(tmpFile))
		
		if self.filesFailed:
			self.filesFailedLoop(output)
		
		return args

	def filesFailedLoop(self, output):
		"""If the user gets an option to abort the commit this is where the prompt should be, otherwise it should notify the user that the pre-commit hook failed and abort the commit."""
		pass
	
	def checkFile(self, fileName):
		"""checkFile will be called for each file that is to be committed."""
		pass

	def filesToCommit(self, args):
		"""Using svn status, find all of the files that the user intends to commit."""
		tmpArgs=list(args)
		tmpArgs.pop(0) # pop the svn path
		tmpArgs.pop(0) # pop "commit" off the list
		flagsWithArguments=('--targets', '-m', '--message', '-F', '--file', '--editor-cmd', '--encoding', '--username', '--password', '--config-dir')
		files=[]
		paths=[]
		while len(tmpArgs)>0:
			current=tmpArgs.pop(0)
			
			if current in flagsWithArguments:
				# pop the argument
				tmpArgs.pop(0)
				continue
			if current[0] == "-":
				continue
				# this isn't a file thats being commited or a path to a working directory.
			#if we made it this far, it must be a list of file to be committed or the path to the working copy
			
			if os.path.isfile(current):
				#collect all files to be committed
				files.append(current)
			elif os.path.isdir(current):
				paths.append(current)
				# this must be a working copy
			
		if len(paths) == 0 and len(files) == 0:
			paths.append('.')
		# need to build the arguments to find out what files codecheck needs to be run on.
		for tmp in paths:	
			tmpArgs=(args[0], '--non-interactive', 'status', tmp)
			for tmpFile in os.popen2(tmpArgs)[1].readlines(): 
				status = tmpFile[0]
				if status == "?":
					#skip files that aren't in SVN
					continue
				# the first character (svn status) and \n and strip whitespace
				tmpFile=tmpFile[1:-1].strip()
				files.append(tmpFile)
		return files
