#!/usr/bin/python
LICENSE="""
The MIT License

Copyright (c) 2008 Asa Ayers <Asa.Ayers@Gmail.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import sys, os, commands
import popen2, select, ConfigParser



class SVN:
	shortcuts=[]
	module_priority=[]
	real_svn=''
	modules={}
	verbose=False
	test_mode=False
	abort=False
	config=None
	
	def __init__(self):
		self.loadConfig()

	def reconstructArguments(self, args, add_quotes=True):
		"""Reconstruct the arguments to they can be copied and pasted.
		This was designed for the shortcuts menu so you can copy and
		paste the converted URLs."""
		ret=[]
		if add_quotes:
			for arg in args:
				if ' ' in arg:
					if '"' in arg:
						quote="'"
					else:
						quote='"'
					arg=quote+arg+quote
				ret.append(arg)
		else:
			ret=args
		return " ".join(ret)
		
	def findSVN(self):
		"""Finds the first svn binary in $PATH that isn't this python file."""
		status, out = commands.getstatusoutput("which -a svn")
		for line in out.split("\n"):
			if line == sys.argv[0]:
				continue
			return line
	
	def loadConfig(self):
		config_folder=os.path.expanduser('~/.svn-extender/')
		exclude_modules=[]
		config = ConfigParser.ConfigParser()
		self.config = config
		config.read([config_folder+'config.ini'])
			
		if config.has_section('options'):
			if config.has_option('options', 'real_svn'):
				self.real_svn=config.get('options', 'real_svn')
			else:
				self.real_svn=self.findSVN()
				
			if config.has_option('options', 'exclude_modules'):
				exclude_modules=config.get('options', 'exclude_modules').split(',')
			
			if config.has_option('options', 'module_priority'):
				self.module_priority=config.get('options', 'module_priority').split(',')
				
			if config.has_option('options', 'verbose'):
				self.verbose=(config.get('options', 'verbose').lower()=='true')
				
			if config.has_option('options', 'test_mode'):
				self.test_mode=(config.get('options', 'test_mode').lower()=='true')
		
		self.printString( ("exclude_modules:", exclude_modules))
		if '*' in exclude_modules:
			return
		
		# theres no reason to load and run the baseModule.
		if 'baseModule' not in exclude_modules:
			exclude_modules.append('baseModule')
		
		modules_dir=config_folder+'modules'
		module_list = os.listdir(modules_dir)
		sys.path.insert(0, modules_dir)
		for module in module_list:
			if os.path.splitext(module)[1].lower() != '.py':
				continue
			short=os.path.splitext(module)[0]
			if short in exclude_modules or short in self.modules:
				continue
			path = modules_dir + "/" + module
			if os.path.isfile(path) and (not module[0] == "."):
				self.printString("Loading Module: " + path)
				self.modules[short]=__import__(short)
		sys.path.remove(modules_dir)
	
	def execute(self, args):
		args[0]=self.real_svn
		args=self.runModules(args)
		
		self.printString("the final command is:")
		self.printString(self.reconstructArguments(args))
		self.printString("========== svn shortcut wrapper ==========")
		if not self.test_mode and not self.abort:
			os.execvp(args[0], args)
		
	def runModules(self, args):
		abort=False
		# run all of the modules up to the star
		while len(self.module_priority)>0:
			module=self.module_priority.pop(0)
			if module == "*":
				break
			self.abort, args=self.runThisModule(args, module)
			if self.abort:
				print "module %s is aborting the command." % (module,)
				return
		
		# Run the remaining modules excluding anything still in the list
		#for module in self.modules:
		offset=0
		while len(self.modules) > len(self.module_priority):
			module=self.modules.keys()[offset]
			if module in self.module_priority:
				offset+=1
				continue
			self.abort, args=self.runThisModule(args, module)
			if self.abort:
				print "module %s is aborting the command." % (module,)
				return
		
		# All that should be left is anything after the star.
		while len(self.module_priority)>0:
			module=self.module_priority.pop(0)
			if module == "*":
				break
			self.abort, args=self.runThisModule(args, module)
			if self.abort:
				print "module %s is aborting the command." % (module,)
				return
		return args
		
	def runThisModule(self, args, module_name):
		obj_processor=self.modules.pop(module_name).__dict__[module_name](self)
		return obj_processor.start(args)
		
	def printString(self, string):
		"""This will only print the string if verbose mode is on."""
		if self.verbose:
			print string

	
if __name__ == '__main__':
	mainModule=SVN()
	mainModule.execute(sys.argv)

